/**
 * Copyright (c) 2000-2005 Liferay, LLC. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portal.servlet;

import java.util.Iterator;
import java.util.List;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.liferay.portal.SystemException;
import com.liferay.portal.service.spring.ThemeServiceUtil;
import com.liferay.portlet.portletaggregatordesigner.service.spring.LayoutTemplateServiceUtil;
import com.liferay.util.Http;
import com.liferay.util.StringPool;
import com.liferay.util.StringUtil;

/**
 * <a href="LayoutTemplateContextListener.java.html"><b><i>View Source</i></b></a>
 * 
 * @author Ivica Cardic
 * @version $Revision: 1.0 $
 * 
 */
public class LayoutTemplateContextListener implements ServletContextListener {
	private static final String LIFERAY_LAYOUT_TEMPLATES_PATH = "/WEB-INF/liferay-layout-templates.xml";
	
	public void contextInitialized(ServletContextEvent sce) {
		try {
			ServletContext ctx = sce.getServletContext();

			_servletContextName = StringUtil.replace(ctx
					.getServletContextName(), StringPool.SPACE,
					StringPool.UNDERLINE);

			_log.debug("Servlet context name is " + _servletContextName);

			String[] xmls = new String[] { Http.URLtoString(ctx
					.getResource(LIFERAY_LAYOUT_TEMPLATES_PATH)) };

			_templateIds = LayoutTemplateServiceUtil.init(_servletContextName,
					xmls);
			Iterator itr = _templateIds.iterator();
			while (itr.hasNext()) {
				String templateId = (String) itr.next();
				String templatePath = LayoutTemplateServiceUtil
						.getLayoutTemplatePath(templateId);
				String content = Http
						.URLtoString(ctx.getResource(templatePath));
				if (content.length() == 0) {
					throw new SystemException("Layout template is not defined");
				}
				LayoutTemplateServiceUtil.setLayoutTemplateContent(templateId,
						content);
			}

		} catch (Exception e2) {
			e2.printStackTrace();
		}
	}

	public void contextDestroyed(ServletContextEvent sce) {
		if (_templateIds != null) {
			Iterator itr = _templateIds.iterator();

			while (itr.hasNext()) {
				String templateId = (String) itr.next();

				try {
					ThemeServiceUtil.uninstallTheme(templateId);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			_templateIds = null;
		}

	}

	private static final Log _log = LogFactory
			.getLog(LayoutTemplateContextListener.class);

	private String _servletContextName;

	private List _templateIds;

}
