/**
 * Copyright (c) 2000-2005 Liferay, LLC. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portlet.portletaggregatordesigner.action;

import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletConfig;
import javax.portlet.PortletPreferences;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.liferay.portal.model.Portlet;
import com.liferay.portal.service.spring.PortletServiceUtil;
import com.liferay.portal.struts.PortletAction;
import com.liferay.portlet.portletaggregatordesigner.model.LayoutTemplate;
import com.liferay.portlet.portletaggregatordesigner.model.LayoutTemplateColumn;
import com.liferay.portlet.portletaggregatordesigner.service.spring.LayoutTemplateServiceUtil;
import com.liferay.portlet.portletaggregatordesigner.util.LayoutTemplateConstants;
import com.liferay.portlet.portletaggregatordesigner.util.LayoutTemplateParamKeys;
import com.liferay.portlet.portletaggregatordesigner.util.LayoutTemplatePrefKeys;
import com.liferay.util.PwdGenerator;
import com.liferay.util.StringPool;
import com.liferay.util.StringUtil;
import com.liferay.util.servlet.SessionMessages;

/**
 * <a href="EditPortletsAction.java.html"><b><i>View Source</i></b></a>
 * 
 * @author Ivica Cardic
 * @version $Revision: 1.0 $
 * 
 */
public class EditPortletsAction extends PortletAction {
	public void processAction(ActionMapping mapping, ActionForm form,
			PortletConfig config, ActionRequest req, ActionResponse res)
			throws Exception {
		PortletPreferences prefs = req.getPreferences();

		String columnId = req.getParameter(LayoutTemplateParamKeys.COLUMN_ID);
		String companyId = req.getParameter(LayoutTemplateParamKeys.COMPANY_ID);
		// String columnType =
		// req.getParameter(LayoutTemplateParamKeys.COLUMN_TYPE);

		String[] portletIds = StringUtil.split(req.getParameter("p_l_"
				+ columnId + "_list"), ",");

		for (int i = 0; i < portletIds.length; i++) {
			if (!portletIds[i].contains(Portlet.INSTANCE_SEPARATOR)) {
				Portlet portlet = PortletServiceUtil.getPortletById(companyId,
						portletIds[i]);
				if (portlet.isInstanceable()) {
					portletIds[i] += Portlet.INSTANCE_SEPARATOR
							+ PwdGenerator.getPassword(4);
				}
			}
		}

		// prefs.setValue(columnId+"-type",columnType);
		prefs.setValues(columnId + LayoutTemplatePrefKeys.PORTLETS, portletIds);
		prefs.store();

		SessionMessages.add(req, config.getPortletName() + ".doEdit");

		res.setRenderParameter(LayoutTemplateParamKeys.COLUMN_ID, columnId);
	}

	public ActionForward render(ActionMapping mapping, ActionForm form,
			PortletConfig config, RenderRequest req, RenderResponse res)
			throws Exception {

		String columnId = req.getParameter(LayoutTemplateParamKeys.COLUMN_ID);

		String layoutTemplateId = req.getPreferences().getValue(
				LayoutTemplatePrefKeys.LAYOUT_TEMPLATE_ID, StringPool.BLANK);
		if (layoutTemplateId.equals(StringPool.BLANK)) {
			return mapping.findForward(getForward(req,
					"portlet.portlet_aggregator_designer.edit"));
		}

		String[] portletIds = req.getPreferences().getValues(
				columnId + LayoutTemplatePrefKeys.PORTLETS, new String[0]);
		if (portletIds == null) {
			portletIds = new String[0];
		}

		req.setAttribute(LayoutTemplateConstants.PORTLET_IDS, portletIds);

		LayoutTemplate lt = LayoutTemplateServiceUtil
				.getLayoutTemplate(layoutTemplateId);
		List columns = lt.getLayoutTemplateColumns();
		for (int i = 0; i < columns.size(); i++) {
			LayoutTemplateColumn ltc = (LayoutTemplateColumn) columns.get(i);
			if (ltc.getName().equals(columnId)) {
				req.setAttribute(LayoutTemplateConstants.COLUMN_TYPE, ltc
						.getType());
				break;
			}
		}

		return mapping.findForward(getForward(req,
				"portlet.portlet_aggregator_designer.edit_portlets"));
	}
}
