/**
 * Copyright (c) 2000-2005 Liferay, LLC. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portlet.portletaggregatordesigner.action;

import java.util.List;

import javax.portlet.ActionRequest;
import javax.portlet.ActionResponse;
import javax.portlet.PortletConfig;
import javax.portlet.PortletPreferences;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.liferay.portal.struts.PortletAction;
import com.liferay.portal.util.Constants;
import com.liferay.portlet.portletaggregatordesigner.model.LayoutTemplateColumn;
import com.liferay.portlet.portletaggregatordesigner.service.spring.LayoutTemplateServiceUtil;
import com.liferay.portlet.portletaggregatordesigner.util.LayoutTemplateParamKeys;
import com.liferay.portlet.portletaggregatordesigner.util.LayoutTemplatePrefKeys;
import com.liferay.util.ParamUtil;
import com.liferay.util.StringPool;
import com.liferay.util.servlet.SessionMessages;

/**
 * <a href="EditPreferencesAction.java.html"><b><i>View Source</i></b></a>
 * 
 * @author Ivica Cardic
 * @version $Revision: 1.0 $
 * 
 */
public class EditPreferencesAction extends PortletAction {

	public void processAction(ActionMapping mapping, ActionForm form,
			PortletConfig config, ActionRequest req, ActionResponse res)
			throws Exception {
		PortletPreferences prefs = req.getPreferences();

		String cmd = ParamUtil.getString(req, Constants.CMD);

		if (!cmd.equals(Constants.UPDATE)) {
			return;
		}

		String portletTitle = ParamUtil.getString(req,
				LayoutTemplateParamKeys.PORTLET_TITLE);
		boolean showPortletBorders = ParamUtil.getBoolean(req,
				LayoutTemplateParamKeys.SHOW_PORTLET_BORDERS);

		String layoutTemplateId = ParamUtil.getString(req,
				LayoutTemplateParamKeys.LAYOUT_TEMPLATE_ID);

		List ltColumns = LayoutTemplateServiceUtil.getLayoutTemplate(
				layoutTemplateId).getLayoutTemplateColumns();

		prefs.setValue(LayoutTemplatePrefKeys.PORTLET_TITLE, portletTitle);
		prefs.setValue(LayoutTemplatePrefKeys.SHOW_PORTLET_BORDERS, Boolean
				.toString(showPortletBorders));

		prefs.setValue(LayoutTemplatePrefKeys.LAYOUT_TEMPLATE_ID,
				layoutTemplateId);

		for (int i = 0; i < ltColumns.size(); i++) {
			LayoutTemplateColumn ltc = (LayoutTemplateColumn) ltColumns.get(i);
			String columnId = ltc.getName();
			String columnType = ltc.getType();
			String curColumnType = prefs.getValue(columnId
					+ LayoutTemplatePrefKeys.TYPE, StringPool.BLANK);
			if (!curColumnType.equals(columnType)) {
				prefs.setValue(columnId + LayoutTemplatePrefKeys.TYPE,
						columnType);
				prefs.setValues(columnId + LayoutTemplatePrefKeys.PORTLETS,
						null);
			}

		}

		prefs.store();

		SessionMessages.add(req, config.getPortletName() + ".doEdit");

	}

	public ActionForward render(ActionMapping mapping, ActionForm form,
			PortletConfig config, RenderRequest req, RenderResponse res)
			throws Exception {

		return mapping.findForward(getForward(req,
				"portlet.portlet_aggregator_designer.edit"));
	}

}