/**
 * Copyright (c) 2000-2005 Liferay, LLC. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portlet.portletaggregatordesigner.action;

import java.util.List;

import javax.portlet.PortletConfig;
import javax.portlet.PortletPreferences;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.servlet.jsp.PageContext;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.liferay.portal.struts.PortletAction;
import com.liferay.portal.util.Constants;
import com.liferay.portlet.portletaggregatordesigner.model.LayoutTemplate;
import com.liferay.portlet.portletaggregatordesigner.model.LayoutTemplateColumn;
import com.liferay.portlet.portletaggregatordesigner.service.spring.LayoutTemplateServiceUtil;
import com.liferay.portlet.portletaggregatordesigner.util.LayoutTemplateConstants;
import com.liferay.portlet.portletaggregatordesigner.util.LayoutTemplatePrefKeys;
import com.liferay.util.GetterUtil;
import com.liferay.util.StringPool;
import com.liferay.util.Validator;

/**
 * <a href="ViewAction.java.html"><b><i>View Source</i></b></a>
 * 
 * @author Ivica Cardic
 * @version $Revision: 1.0 $
 * 
 */
public class ViewAction extends PortletAction {

	public ActionForward render(ActionMapping mapping, ActionForm form,
			PortletConfig config, RenderRequest req, RenderResponse res)
			throws Exception {

		try {
			PortletPreferences prefs = req.getPreferences();

			String portletTitle = prefs.getValue(
					LayoutTemplatePrefKeys.PORTLET_TITLE, StringPool.BLANK);
			boolean showPortletBorders = GetterUtil.getBoolean(prefs.getValue(
					LayoutTemplatePrefKeys.SHOW_PORTLET_BORDERS, "true"));

			if (Validator.isNotNull(portletTitle)) {
				res.setTitle(portletTitle);
			}

			String layoutTemplateId = prefs
					.getValue(LayoutTemplatePrefKeys.LAYOUT_TEMPLATE_ID,
							StringPool.BLANK);

			if (Validator.isNull(layoutTemplateId)
					|| layoutTemplateId.equals(StringPool.BLANK)) {
				return mapping.findForward("/portal/portlet_not_setup");
			}

			LayoutTemplate layoutTemplate = LayoutTemplateServiceUtil
					.getLayoutTemplate(layoutTemplateId);
			List ltcList = layoutTemplate.getLayoutTemplateColumns();

			boolean exist = false;
			for (int i = 0; i < ltcList.size(); i++) {
				LayoutTemplateColumn ltc = (LayoutTemplateColumn) ltcList
						.get(i);
				String[] portlets = prefs.getValues(ltc.getName()
						+ LayoutTemplatePrefKeys.PORTLETS, new String[0]);
				if (portlets != null && portlets.length > 0) {
					exist = true;
					break;
				}
			}

			if (exist) {
				req.setAttribute(LayoutTemplateConstants.LAYOUT_REMPLATE,
						layoutTemplate);
			} else {
				prefs
						.setValue(LayoutTemplatePrefKeys.SHOW_PORTLET_BORDERS,
								"1");
				return mapping.findForward("/portal/portlet_not_setup");
			}

			if (showPortletBorders) {
				return mapping
						.findForward("portlet.portlet_aggregator_designer.view_1");
			} else {
				return mapping
						.findForward("portlet.portlet_aggregator_designer.view_2");
			}
		} catch (Exception e) {
			req.setAttribute(PageContext.EXCEPTION, e);

			return mapping.findForward(Constants.COMMON_ERROR);
		}
	}

}