/**
 * Copyright (c) 2000-2005 Liferay, LLC. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portlet.portletaggregatordesigner.service.impl;

import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import com.liferay.portal.SystemException;
import com.liferay.portal.model.Portlet;
import com.liferay.portal.service.impl.PrincipalBean;
import com.liferay.portal.util.EntityResolver;
import com.liferay.portal.util.ReleaseInfo;
import com.liferay.portlet.portletaggregatordesigner.model.LayoutTemplate;
import com.liferay.portlet.portletaggregatordesigner.model.LayoutTemplateColumn;
import com.liferay.portlet.portletaggregatordesigner.service.spring.LayoutTemplateService;
import com.liferay.portlet.portletaggregatordesigner.util.LayoutTemplateUtil;
import com.liferay.portlet.portletaggregatordesigner.util.LayoutTemplateXMLConstants;
import com.liferay.util.CollectionFactory;
import com.liferay.util.GetterUtil;
import com.liferay.util.ListUtil;
import com.liferay.util.SimpleCachePool;
import com.liferay.util.StringPool;

/**
 * <a href="LayoutTemplateServiceImpl.java.html"><b><i>View Source</i></b></a>
 * 
 * @author Ivica Cardic
 * @version $Revision: 1.0 $
 * 
*/
public class LayoutTemplateServiceImpl extends PrincipalBean implements
		LayoutTemplateService {

	// Business methods

	public LayoutTemplate getLayoutTemplate(String layoutTemplateId)
			throws SystemException {
		LayoutTemplate layoutTemplate = (LayoutTemplate) _getLayoutTemplatesPool()
				.get(layoutTemplateId);

		return layoutTemplate;
	}

	public String getLayoutTemplatePath(String layoutTemplateId)
			throws SystemException {
		LayoutTemplate layoutTemplate = (LayoutTemplate) _getLayoutTemplatesPool()
				.get(layoutTemplateId);

		return layoutTemplate.getTemplatePath();
	}

	public void setLayoutTemplateContent(String layoutTemplateId, String content)
			throws SystemException {
		LayoutTemplate layoutTemplate = (LayoutTemplate) _getLayoutTemplatesPool()
				.get(layoutTemplateId);
		layoutTemplate.setContent(content);
		layoutTemplate
				.setLayoutTemplateColumns(_getLayoutTemplateColumns(content));

	}

	private List _getLayoutTemplateColumns(String templateContent)
			throws SystemException {
		int x = 0;
		int y = templateContent
				.indexOf(LayoutTemplateUtil.RUNTIME_PORTLET_COLUMN_OPEN);

		List templateColumns = new ArrayList();

		while (y != -1) {
			int close1 = templateContent.indexOf(
					LayoutTemplateUtil.RUNTIME_PORTLET_COLUMN_CLOSE_1, y);
			int close2 = templateContent.indexOf(
					LayoutTemplateUtil.RUNTIME_PORTLET_COLUMN_CLOSE_2, y);

			if ((close2 == -1) || ((close1 != -1) && (close1 < close2))) {
				x = close1
						+ LayoutTemplateUtil.RUNTIME_PORTLET_COLUMN_CLOSE_1
								.length();
			} else {
				x = close2
						+ LayoutTemplateUtil.RUNTIME_PORTLET_COLUMN_CLOSE_2
								.length();
			}

			String runtimePortletColumn = templateContent.substring(y, x);

			_log.debug("Parsing [" + runtimePortletColumn + "]");

			SAXReader reader = new SAXReader();

			Document doc;
			try {
				doc = reader.read(new StringReader(runtimePortletColumn));
			} catch (DocumentException e) {
				throw new SystemException(e);
			}

			Element root = doc.getRootElement();

			String name = root.attributeValue(LayoutTemplateXMLConstants.NAME);
			String width = root.attributeValue(LayoutTemplateXMLConstants.WIDTH);
			String type = root.attributeValue(LayoutTemplateXMLConstants.TYPE);

			if (name == null || name.equals(StringPool.BLANK)) {
				throw new SystemException("column \"name\" cannot be empty");
			}
			if (name != null
					&& (name.equals("n1") || name.equals("n2") || name
							.equals("w1"))) {
				throw new SystemException(
						"column \"name\" cannot have naems \"n1\" or \"n2\" or \"w1\"");
			}

			if (width == null || width.equals(StringPool.BLANK)) {
				throw new SystemException("column \"width\" cannot be empty");
			}
			if (type == null || type.equals(StringPool.BLANK)) {
				throw new SystemException("column \"type\" cannot be empty");
			}
			if (type != null && !(type.equals("n") || type.equals("w"))) {
				throw new SystemException(
						"column \"type\" must be \"n\" or \"w\"");
			}

			templateColumns.add(new LayoutTemplateColumn(name, width, type));

			y = templateContent.indexOf(
					LayoutTemplateUtil.RUNTIME_PORTLET_COLUMN_OPEN, x);
		}
		return templateColumns;
	}

	public String getLayoutTemplateContent(String layoutTemplateId)
			throws SystemException {
		LayoutTemplate layoutTemplate = (LayoutTemplate) _getLayoutTemplatesPool()
				.get(layoutTemplateId);

		return layoutTemplate.getContent();

	}

	public List getLayoutTemplates() {
		List layoutTemplates = ListUtil
				.fromCollection(_getLayoutTemplatesPool().values());

		Collections.sort(layoutTemplates);

		return layoutTemplates;
	}

	public List init(String[] xmls) {
		return init(null, xmls);
	}

	public List init(String servletContextName, String[] xmls) {
		List layoutTemplateIds = new ArrayList();

		Map layoutTemplatePool = _getLayoutTemplatesPool();
		layoutTemplatePool.clear();

		try {
			for (int i = 0; i < xmls.length; i++) {
				Iterator itr = _readLayoutTemplateConfig(servletContextName,
						xmls[i], layoutTemplatePool).iterator();

				while (itr.hasNext()) {
					String layoutTemplateId = (String) itr.next();

					if (!layoutTemplateIds.contains(layoutTemplateId)) {
						layoutTemplateIds.add(layoutTemplateId);
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}

		return layoutTemplateIds;
	}

	public void uninstallLayoutTemplate(String layoutTemplateId) {
		_getLayoutTemplatesPool().remove(layoutTemplateId);
	}

	// Private methods

	private Map _getLayoutTemplatesPool() {
		String layoutTemplatesPoolKey = LayoutTemplateServiceImpl.class
				.getName()
				+ ".LAYOUT_TEMPLATES";

		Map layoutTemplatesPool = (Map) SimpleCachePool
				.get(layoutTemplatesPoolKey);

		if (layoutTemplatesPool == null) {
			layoutTemplatesPool = CollectionFactory.getSyncHashMap();

			SimpleCachePool.put(layoutTemplatesPoolKey, layoutTemplatesPool);
		}

		return layoutTemplatesPool;
	}

	private Set _readLayoutTemplateConfig(String servletContextName,
			String xml, Map layoutTemplatesPool) throws DocumentException,
			IOException {

		Set templateIds = new HashSet();

		if (xml == null) {
			return templateIds;
		}

		// IVICAC should be true
		SAXReader reader = new SAXReader(false);
		reader.setEntityResolver(new EntityResolver());

		Document doc = reader.read(new StringReader(xml));

		Element root = doc.getRootElement();

		Set compatibleVersions = new HashSet();

		Element compatibilityEl = root.element(LayoutTemplateXMLConstants.COMPATIBILITY);

		if (compatibilityEl != null) {
			Iterator itr = compatibilityEl.elements(LayoutTemplateXMLConstants.VERSION)
					.iterator();

			while (itr.hasNext()) {
				Element versionEl = (Element) itr.next();

				String version = versionEl.getTextTrim();

				compatibleVersions.add(version);
			}
		}

		if (!compatibleVersions.contains(ReleaseInfo.getVersion())) {
			_log.error("Layout templates in this WAR are not compatible with "
					+ ReleaseInfo.getServerInfo());

			return templateIds;
		}

		Iterator itr1 = root.elements(LayoutTemplateXMLConstants.TEMPLATE).iterator();

		while (itr1.hasNext()) {
			Element template = (Element) itr1.next();

			String templateId = template.attributeValue(LayoutTemplateXMLConstants.ID);
			if (servletContextName != null) {
				templateId = templateId + Portlet.WAR_SEPARATOR
						+ servletContextName;
			}

			templateIds.add(templateId);

			LayoutTemplate templateModel = (LayoutTemplate) layoutTemplatesPool
					.get(templateId);

			if (templateModel == null) {
				templateModel = new LayoutTemplate(templateId);

				layoutTemplatesPool.put(templateId, templateModel);
			}

			if (servletContextName != null) {
				templateModel.setServletContextName(servletContextName);
			}

			templateModel.setName(GetterUtil.get(template
					.attributeValue(LayoutTemplateXMLConstants.NAME), templateModel
					.getName()));
			templateModel.setTemplatePath(GetterUtil.get(template
					.elementText(LayoutTemplateXMLConstants.TEMPLATE_PATH), templateModel
					.getTemplatePath()));

			Element settingsEl = template.element(LayoutTemplateXMLConstants.SETTINGS);

			if (settingsEl != null) {
				Iterator itr2 = settingsEl.elements(LayoutTemplateXMLConstants.SETTING)
						.iterator();

				while (itr2.hasNext()) {
					Element settingEl = (Element) itr2.next();

					String key = settingEl.attributeValue(LayoutTemplateXMLConstants.KEY);
					String value = settingEl
							.attributeValue(LayoutTemplateXMLConstants.VALUE);

					templateModel.setSetting(key, value);
				}
			}
		}

		return templateIds;
	}

	private static final Log _log = LogFactory
			.getLog(LayoutTemplateServiceImpl.class);
}
