/**
 * Copyright (c) 2000-2005 Liferay, LLC. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package com.liferay.portlet.portletaggregatordesigner.util;

import java.io.IOException;
import java.io.StringReader;
import java.util.List;

import javax.portlet.PortletConfig;
import javax.portlet.PortletPreferences;
import javax.portlet.RenderRequest;
import javax.portlet.RenderResponse;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import com.liferay.portal.PortalException;
import com.liferay.portal.SystemException;
import com.liferay.portal.model.Layout;
import com.liferay.portal.model.LayoutTypePortlet;
import com.liferay.portal.model.Portlet;
import com.liferay.portal.service.persistence.PortletPreferencesPK;
import com.liferay.portal.service.spring.PortletPreferencesServiceUtil;
import com.liferay.portal.service.spring.PortletServiceUtil;
import com.liferay.portal.theme.PortletDisplay;
import com.liferay.portal.theme.PortletDisplayFactory;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portal.util.PortalUtil;
import com.liferay.portal.util.PortletKeys;
import com.liferay.portal.util.WebKeys;
import com.liferay.portlet.journalcontent.util.RuntimePortletUtil;
import com.liferay.portlet.portletaggregatordesigner.model.LayoutTemplate;
import com.liferay.portlet.portletaggregatordesigner.model.LayoutTemplateColumn;
import com.liferay.portlet.portletaggregatordesigner.service.spring.LayoutTemplateServiceUtil;
import com.liferay.util.StringPool;
import com.liferay.util.Validator;

/**
 * <a href="LayoutTemplateUtil.java.html"><b><i>View Source</i></b></a>
 * 
 * @author Ivica Cardic
 * @version $Revision: 1.0 $
 * 
 */
public class LayoutTemplateUtil {
	private final static Log _log = LogFactory.getLog(LayoutTemplateUtil.class);

	public static final String RUNTIME_PORTLET_COLUMN_OPEN = "<runtime-portlet-column";

	public static final String RUNTIME_PORTLET_COLUMN_CLOSE_1 = "</runtime-portlet-column>";

	public static final String RUNTIME_PORTLET_COLUMN_CLOSE_2 = "/>";

	public static final String RUNTIME_PORTLET_LIST_OPEN = "<runtime-portlet-list";

	public static final String RUNTIME_PORTLET_LIST_CLOSE_1 = "</runtime-portlet-list>";

	public static final String RUNTIME_PORTLET_LIST_CLOSE_2 = "/>";

	public static void processEditContent(PageContext pageContext,
			RenderRequest renderRequest, RenderResponse renderResponse,
			LayoutTemplate layoutTemplate) throws Exception {
		String templateContent = layoutTemplate.getContent();

		JspWriter out = pageContext.getOut();
		HttpServletRequest req = (HttpServletRequest) pageContext.getRequest();
		HttpServletResponse res = (HttpServletResponse) pageContext
				.getResponse();
		PortletPreferences prefs = renderRequest.getPreferences();

		templateContent = templateContent
				.replace("<?xml version=\"1.0\"?>", "");

		int x = 0;
		int y = templateContent.indexOf(RUNTIME_PORTLET_COLUMN_OPEN);

		while (y != -1) {
			out.print(templateContent.substring(x, y));
			out.flush();

			// <runtime-portlet name="19" width="200"></runtime-portlet>
			// <runtime-portlet name="19" width="200"/>

			// <runtime-portlet-column name="column_1"
			// type="n" width="200"></runtime-portlet-column>
			// <runtime-portlet-column name="column_1" type="n" width="200"/>

			// <runtime-portlet-list
			// column-name="column_1"></runtime-portlet-column>
			// <runtime-portlet-column column-name="column_1"/>

			int close1 = templateContent.indexOf(
					RUNTIME_PORTLET_COLUMN_CLOSE_1, y);
			int close2 = templateContent.indexOf(
					RUNTIME_PORTLET_COLUMN_CLOSE_2, y);

			if ((close2 == -1) || ((close1 != -1) && (close1 < close2))) {
				x = close1 + RUNTIME_PORTLET_COLUMN_CLOSE_1.length();
			} else {
				x = close2 + RUNTIME_PORTLET_COLUMN_CLOSE_2.length();
			}

			String runtimePortletColumn = templateContent.substring(y, x);

			_log.debug("Parsing [" + runtimePortletColumn + "]");

			SAXReader reader = new SAXReader();

			Document doc = reader.read(new StringReader(runtimePortletColumn));

			Element root = doc.getRootElement();

			String columnId = root.attributeValue(LayoutTemplateXMLConstants.NAME);
			String width = root.attributeValue(LayoutTemplateXMLConstants.WIDTH);
			String columnType = root.attributeValue(LayoutTemplateXMLConstants.TYPE); // n or
			// w

			req.setAttribute(LayoutTemplateConstants.COLUMN_ID, columnId);
			req.setAttribute(LayoutTemplateConstants.COLUMN_WIDTH, width);
			req.setAttribute(LayoutTemplateConstants.COLUMN_TYPE, columnType);

			String curColumnType = prefs.getValue(columnId + LayoutTemplatePrefKeys.TYPE,
					StringPool.BLANK);
			String[] portletIds = prefs.getValues(columnId
					+ LayoutTemplatePrefKeys.PORTLETS, new String[0]);
			if (portletIds == null) {
				portletIds = new String[0];
			}

			if (!curColumnType.equals(columnType)) {
				portletIds = new String[0];
			}

			req.setAttribute(LayoutTemplateConstants.PORTLET_IDS, portletIds);

			//
			_includeViewPortletsColumn(req, res);

			y = templateContent.indexOf(RUNTIME_PORTLET_COLUMN_OPEN, x);
		}

		if (y == -1) {
			out.print(templateContent.substring(x, templateContent.length()));
		}

	}

	public static boolean hasPortletId(String columnId, String portletId,
			RenderRequest renderRequest) {
		String[] portletIds = renderRequest.getPreferences().getValues(
				columnId + LayoutTemplatePrefKeys.PORTLETS, new String[0]);
		if (portletIds == null) {
			return false;
		}

		for (int i = 0; i < portletIds.length; i++) {
			if (portletIds[i].equals(portletId)) {
				return true;
			}
		}
		return false;

	}

	public static String getPortletId(String portletId) {
		return StringUtils.substringBefore(portletId,
				Portlet.INSTANCE_SEPARATOR);
	}

	public static void processViewContent(PageContext pageContext,
			ServletContext ctx, RenderRequest renderRequest,
			RenderResponse renderResponse, ThemeDisplay themeDisplay,
			LayoutTemplate layoutTemplate) throws Exception {

		String templateContent = layoutTemplate.getContent();

		JspWriter out = pageContext.getOut();
		HttpServletRequest req = (HttpServletRequest) pageContext.getRequest();
		HttpServletResponse res = (HttpServletResponse) pageContext
				.getResponse();

		// Get current portlet config

		PortletConfig portletConfig = (PortletConfig) req
				.getAttribute(WebKeys.JAVAX_PORTLET_CONFIG);

		// Clone portlet display

		PortletDisplay portletDisplay = themeDisplay.getPortletDisplay();

		PortletDisplay portletDisplayClone = (PortletDisplay) portletDisplay
				.clone();

		// Process content

		req.setAttribute(LayoutTemplateConstants.PORTLET_INSIDE_PORTLET_AGGREGATOR,
				Boolean.TRUE);

		//List allPortletIds = new ArrayList();
		//req.setAttribute(LayoutTemplateConstants.ALL_PORTLET_IDS, allPortletIds);

		templateContent = templateContent
				.replace("<?xml version=\"1.0\"?>", "");

		try {
			int x = 0;
			int y = templateContent
					.indexOf(RuntimePortletUtil.RUNTIME_PORTLET_OPEN);

			while (y != -1) {
				out.print(templateContent.substring(x, y));
				out.flush();

				// The XSL template engine may or may not close the
				// runtime-portlet tag that the user inputs. The processor must
				// be prepared to deal with both scenarios.

				// <runtime-portlet name="19" width="200"></runtime-portlet>
				// <runtime-portlet name="19" width="200"/>

				int close1 = -1;
				int close2 = -1;
				int closeTag1Length = -1;
				int closeTag2Length = -1;
				boolean proccesPortlet = false;
				boolean proccesPortletList = false;
				boolean proccesPortletColumn = false;

				String pattern = templateContent.substring(y, y
						+ RUNTIME_PORTLET_COLUMN_OPEN.length());
				if (pattern.startsWith(RUNTIME_PORTLET_COLUMN_OPEN)) {
					close1 = templateContent.indexOf(
							RUNTIME_PORTLET_COLUMN_CLOSE_1, y);
					closeTag1Length = RUNTIME_PORTLET_COLUMN_CLOSE_1.length();
					close2 = templateContent.indexOf(
							RUNTIME_PORTLET_COLUMN_CLOSE_2, y);
					closeTag2Length = RUNTIME_PORTLET_COLUMN_CLOSE_2.length();
					proccesPortletColumn = true;
				} else if (pattern.startsWith(RUNTIME_PORTLET_LIST_OPEN)) {
					close1 = templateContent.indexOf(
							RUNTIME_PORTLET_LIST_CLOSE_1, y);
					closeTag1Length = RUNTIME_PORTLET_LIST_CLOSE_1.length();
					close2 = templateContent.indexOf(
							RUNTIME_PORTLET_LIST_CLOSE_2, y);
					closeTag2Length = RUNTIME_PORTLET_LIST_CLOSE_2.length();
					proccesPortletList = true;
				} else if (pattern
						.startsWith(RuntimePortletUtil.RUNTIME_PORTLET_OPEN)) {
					close1 = templateContent.indexOf(
							RuntimePortletUtil.RUNTIME_PORTLET_CLOSE_1, y);
					closeTag1Length = RuntimePortletUtil.RUNTIME_PORTLET_CLOSE_1
							.length();
					close2 = templateContent.indexOf(
							RuntimePortletUtil.RUNTIME_PORTLET_CLOSE_2, y);
					closeTag2Length = RuntimePortletUtil.RUNTIME_PORTLET_CLOSE_2
							.length();
					proccesPortlet = true;
				}

				if ((close2 == -1) || ((close1 != -1) && (close1 < close2))) {
					x = close1 + closeTag1Length;
				} else {
					x = close2 + closeTag2Length;
				}

				String runtimeTag = templateContent.substring(y, x);

				_log.debug("Parsing [" + runtimeTag + "]");

				if (proccesPortlet) {
					_processPortlet(runtimeTag, ctx, req, res, pageContext,
							portletConfig, renderRequest, renderResponse,
							themeDisplay);
				}
				if (proccesPortletList) {
					_processPortletSelectList(runtimeTag, req, res,
							layoutTemplate.getLayoutTemplateColumns());
				}
				if (proccesPortletColumn) {
					_processPortletColumn(runtimeTag, /*allPortletIds,*/ ctx, req,
							res, pageContext, portletConfig, renderRequest,
							renderResponse, themeDisplay);
				}

				y = templateContent.indexOf(
						RuntimePortletUtil.RUNTIME_PORTLET_OPEN, x);
			}

			if (y == -1) {
				out.print(templateContent
						.substring(x, templateContent.length()));
				out.flush();
			}
		} finally {

			// Copy portlet display and recycle the clone

			portletDisplay.copyFrom(portletDisplayClone);

			try {
				PortletDisplayFactory.recycle(portletDisplayClone);
			} catch (Exception e) {
				_log.error(e.getMessage());
			}
		}

	}

	private static void _processPortlet(String runtimePortlet,
			ServletContext ctx, HttpServletRequest req,
			HttpServletResponse res, PageContext pageContext,
			PortletConfig portletConfig, RenderRequest renderRequest,
			RenderResponse renderResponse, ThemeDisplay themeDisplay)
			throws DocumentException, SystemException, NumberFormatException,
			IOException, ServletException {

		SAXReader reader = new SAXReader();

		Document doc = reader.read(new StringReader(runtimePortlet));

		Element root = doc.getRootElement();

		String portletId = root.attributeValue(LayoutTemplateXMLConstants.NAME);
		String instanceId = root.attributeValue(LayoutTemplateXMLConstants.INSTANCE);
		String width = root.attributeValue(LayoutTemplateXMLConstants.WIDTH);

		// For now, do not nest other Journal Content or Portlet
		// Aggregator or Portlet Aggregator Designer portlets

		if (!portletId.startsWith(PortletKeys.JOURNAL_CONTENT_NARROW)
				&& !portletId.startsWith(PortletKeys.JOURNAL_CONTENT_WIDE)
				&& !portletId.startsWith(PortletKeys.PORTLET_AGGREGATOR_NARROW)
				&& !portletId.startsWith(PortletKeys.PORTLET_AGGREGATOR_WIDE)
				&& !portletId
						.startsWith(LayoutTemplatePortletKeys.PORTLET_AGGREGATOR_DESIGNER_NARROW)
				&& !portletId
						.startsWith(LayoutTemplatePortletKeys.PORTLET_AGGREGATOR_DESIGNER_WIDE)) {

			Portlet portlet = PortletServiceUtil.getPortletById(themeDisplay
					.getCompanyId(), portletId);

			// If the portlet is instanceable, it must have a valid
			// instance id

			if (portlet.isInstanceable()) {
				if (Validator.isNotNull(instanceId)
						&& Validator.isPassword(instanceId)
						&& (instanceId.length() == 4)) {

					portletId += Portlet.INSTANCE_SEPARATOR + instanceId;

					portlet = PortletServiceUtil.getPortletById(themeDisplay
							.getCompanyId(), portletId);
				} else {
					_log.debug("Portlet " + portlet.getPortletId()
							+ " is instanceable but does not have a "
							+ "valid instance id");

					portlet = null;
				}
			}

			// Render the portlet if and only if it is not null

			if (portlet != null) {
				PortalUtil.renderPortlet(ctx, req, res, portlet, new Integer(
						width), StringPool.BLANK, new Integer(0),
						new Integer(0));

			}
		}

		_defineObjects(req, portletConfig, renderRequest, renderResponse);

	}

	private static void _processPortletSelectList(String runtimePortletList,
			HttpServletRequest req, HttpServletResponse res,
			List layoutTemplateColumns) throws SystemException,
			ServletException, IOException, DocumentException {

		SAXReader reader = new SAXReader();

		Document doc = reader.read(new StringReader(runtimePortletList));

		Element root = doc.getRootElement();

		String columnName = root.attributeValue(LayoutTemplateXMLConstants.COLUMN_NAME);

		LayoutTemplateColumn ltc = _getLayoutTemplateColumn(
				layoutTemplateColumns, columnName);

		// Render the portlet list
		_renderPortletSelectList(req, res, columnName, ltc.getType(), ltc
				.getWidth());
	}

	private static LayoutTemplateColumn _getLayoutTemplateColumn(
			List layoutTemplateColumns, String columnId) {
		for (int i = 0; i < layoutTemplateColumns.size(); i++) {
			LayoutTemplateColumn ltc = (LayoutTemplateColumn) layoutTemplateColumns
					.get(i);
			if (ltc.getName().equals(columnId)) {
				return ltc;
			}
		}
		return null;
	}

	private static LayoutTemplate _getLayoutTemplate(List layoutTemplates,
			String layoutTemplateId) {
		for (int i = 0; i < layoutTemplates.size(); i++) {
			LayoutTemplate lt = (LayoutTemplate) layoutTemplates.get(i);
			if (lt.getLayoutTemplateId().equals(layoutTemplateId)) {
				return lt;
			}
		}
		return null;
	}

	private static void _processPortletColumn(String runtimePortletColumn,
			/*List allPortletIds,*/ ServletContext ctx, HttpServletRequest req,
			HttpServletResponse res, PageContext pageContext,
			PortletConfig portletConfig, RenderRequest renderRequest,
			RenderResponse renderResponse, ThemeDisplay themeDisplay)
			throws DocumentException, IOException, SystemException,
			NumberFormatException, ServletException {

		JspWriter out = pageContext.getOut();

		SAXReader reader = new SAXReader();

		Document doc = reader.read(new StringReader(runtimePortletColumn));

		Element root = doc.getRootElement();

		String columnId = root.attributeValue(LayoutTemplateXMLConstants.NAME);
		String columnWidth = root.attributeValue(LayoutTemplateXMLConstants.WIDTH);
		String columnType = root.attributeValue(LayoutTemplateXMLConstants.TYPE);

		req.setAttribute(LayoutTemplateConstants.COLUMN_ID, columnId);
		req.setAttribute(LayoutTemplateConstants.COLUMN_TYPE, columnType);
		req.setAttribute(LayoutTemplateConstants.COLUMN_WIDTH, columnWidth);

		PortletPreferences prefs = renderRequest.getPreferences();

		String[] portletIds = prefs.getValues(columnId + LayoutTemplatePrefKeys.PORTLETS,
				new String[0]);
		if (portletIds == null) {
			portletIds = new String[0];
		}

		//allPortletIds.addAll(Arrays.asList(portletIds));

		out.print("<div id=\"layout-column_" + renderResponse.getNamespace()
				+ columnId + "\">");

		if (portletIds != null) {
			for (int i = 0; i < portletIds.length; i++) {
				Portlet portlet = PortletServiceUtil.getPortletById(
						themeDisplay.getCompanyId(), portletIds[i]);

				// Render the portlet if and only if it is not null

				if (portlet != null) {
					PortalUtil.renderPortlet(ctx, req, res, pageContext,
							portlet, new Integer(columnWidth), renderResponse
									.getNamespace()
									+ columnId, new Integer(i), new Integer(
									portletIds.length));
				}

				_defineObjects(req, portletConfig, renderRequest,
						renderResponse);
			}
		}

		out.print("</div>");
		out.flush();
	}

	public static void movePortletUp(HttpServletRequest req, Layout layout,
			String portletAggregatorId, String columnId, String portletId)
			throws Exception {
		PortletPreferences pp = _getPortletPreferences(req, layout,
				portletAggregatorId);
		String[] portletIds = pp.getValues(columnId + LayoutTemplatePrefKeys.PORTLETS,
				new String[0]);

		if (portletIds == null || portletIds.length == 0) {
			return;
		}

		int index = -1;
		for (int i = 0; i < portletIds.length; i++) {
			if (portletIds[i].equals(portletId)) {
				index = i;
				break;
			}
		}
		if (index != -1) {
			portletIds[index] = portletIds[(index + 1) % portletIds.length];
			portletIds[(index + 1) % portletIds.length] = portletId;

			pp.setValues(columnId + LayoutTemplatePrefKeys.PORTLETS, portletIds);
			pp.store();
		}
	}

	public static void removePortlet(HttpServletRequest req, Layout layout,
			String portletAggregatorId, String columnId, String portletId)
			throws Exception {
		PortletPreferences prefs = _getPortletPreferences(req, layout,
				portletAggregatorId);
		String[] portletIds = prefs.getValues(columnId + LayoutTemplatePrefKeys.PORTLETS,
				new String[0]);

		if (portletIds == null || portletIds.length == 0) {
			return;
		}

		String[] tempPortletIds = portletIds;
		portletIds = new String[portletIds.length - 1];

		if (portletIds.length > 0) {
			int j = 0;
			for (int i = 0; i < tempPortletIds.length; i++) {
				if (tempPortletIds[i].equals(portletId)) {
					continue;
				} else {
					portletIds[j++] = tempPortletIds[i];
				}
			}
		}
		prefs.setValues(columnId + LayoutTemplatePrefKeys.PORTLETS, portletIds);
		prefs.store();
	}

	public static void movePortletDown(HttpServletRequest req, Layout layout,
			String portletAggregatorId, String columnId, String portletId)
			throws Exception {
		PortletPreferences prefs = _getPortletPreferences(req, layout,
				portletAggregatorId);
		String[] portletIds = prefs.getValues(columnId + LayoutTemplatePrefKeys.PORTLETS,
				new String[0]);
		if (portletIds == null) {
			portletIds = new String[0];
		}

		int index = -1;
		for (int i = 0; i < portletIds.length; i++) {
			if (portletIds[i].equals(portletId)) {
				index = i;
				break;
			}
		}
		if (index != -1) {
			portletIds[index] = portletIds[portletIds.length - 1 - index];
			portletIds[portletIds.length - 1 - index] = portletId;

			prefs.setValues(columnId + LayoutTemplatePrefKeys.PORTLETS, portletIds);
			prefs.store();
		}
	}

	private static PortletPreferences _getPortletPreferences(
			HttpServletRequest req, Layout layout, String portletAggregatorId)
			throws SystemException, PortalException {

		PortletPreferencesPK ppK = PortalUtil.getPortletPreferencesPK(req,
				portletAggregatorId);
		return PortletPreferencesServiceUtil.getPreferences(layout
				.getCompanyId(), ppK);
	}

	private static void _defineObjects(HttpServletRequest req,
			PortletConfig portletConfig, RenderRequest renderRequest,
			RenderResponse renderResponse) {

		req.setAttribute(WebKeys.JAVAX_PORTLET_CONFIG, portletConfig);
		req.setAttribute(WebKeys.JAVAX_PORTLET_REQUEST, renderRequest);
		req.setAttribute(WebKeys.JAVAX_PORTLET_RESPONSE, renderResponse);
	}

	private static void _includeViewPortletsColumn(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		RequestDispatcher rd = request
				.getRequestDispatcher("/html/portlet/portlet_aggregator_designer/view_portlets_column.jsp");

		rd.include(request, response);

	}

	private static void _renderPortletSelectList(HttpServletRequest request,
			HttpServletResponse response, String columnId, String columnType,
			String columnWidth) throws SystemException, ServletException,
			IOException {

		request.setAttribute("columnId", columnId);
		request.setAttribute("columnType", columnType);
		request.setAttribute("columnWidth", columnWidth);

		RequestDispatcher rd = request
				.getRequestDispatcher("/html/portlet/portlet_aggregator_designer/portlet_select_list.jsp");
		rd.include(request, response);
	}

	public static boolean isPortletInsidePortletAggregator(
			HttpServletRequest req) {
		Boolean value = (Boolean) req
				.getAttribute(LayoutTemplateConstants.PORTLET_INSIDE_PORTLET_AGGREGATOR);
		if (value == null) {
			return false;
		}
		return value.booleanValue();
	}

	public static boolean checkPortlet(HttpServletRequest req,
			LayoutTypePortlet layoutTypePortlet, String companyId,
			String portletId) throws PortalException, SystemException {
		if (layoutTypePortlet.hasPortletId(portletId)) {
			return true;
		}

		List layoutTemplates = LayoutTemplateServiceUtil.getLayoutTemplates();

		Portlet[] portlets = layoutTypePortlet.getPortlets();
		for (int i = 0; i < portlets.length; i++) {
			Portlet portlet = portlets[i];
			if (portlet.getPortletId().startsWith("77")
					|| portlet.getPortletId().startsWith("78")) {
				PortletPreferencesPK ppK = PortalUtil.getPortletPreferencesPK(
						req, portlet.getPortletId());
				PortletPreferences pref = PortletPreferencesServiceUtil
						.getPreferences(companyId, ppK);

				String layoutTemplateId = pref.getValue(
						LayoutTemplatePrefKeys.LAYOUT_TEMPLATE_ID, StringPool.BLANK);

				if (layoutTemplateId != null
						&& !layoutTemplateId.equals(StringPool.BLANK)) {
					LayoutTemplate layoutTemplate = _getLayoutTemplate(
							layoutTemplates, layoutTemplateId);

					List layoutTemplateColumns = layoutTemplate
							.getLayoutTemplateColumns();

					if (layoutTemplateColumns != null
							&& layoutTemplateColumns.size() > 0) {
						for (int j = 0; j < layoutTemplateColumns.size(); j++) {
							LayoutTemplateColumn layoutTemplateColumn = (LayoutTemplateColumn) layoutTemplateColumns
									.get(j);
							String[] portletIds = pref.getValues(
									layoutTemplateColumn.getName()
											+ LayoutTemplatePrefKeys.PORTLETS,
									new String[0]);
							if (portletIds == null) {
								portletIds = new String[0];
							}
							for (int k = 0; k < portletIds.length; k++) {
								if (portletIds[k] != null
										&& portletIds[k].equals(portletId)) {
									return true;
								}
							}
						}
					}
				}
			}
		}

		return false;
	}
}
